<?php

/**
 * Este arquivo deve ficar fora do acesso publico da API
 **/

// Bloqueia o acesso direto ao arquivo  
substr_compare($_SERVER['SCRIPT_NAME'], basename(__FILE__), -strlen(basename(__FILE__))) === 0 ? die("Acesso direto não permitido. #003"):FALSE;

date_default_timezone_set('America/Sao_Paulo');

/************************************************************************/
/**
 * Just to see better data output
 **/
function printArray($data){
    
    echo "<pre>";
    print_r($data);
    echo "</pre>";
    
    writeToLog($data, "print");
    
}

/**
 * Write data to log file.
 *
 * @param mixed $data
 * @param string $title
 * @return bool
 */
function writeToLog($data, $title, $print = 1)
{
	$log = "\n------------------------\n";
	$log .= date("Y.m.d G:i:s")."\n";
	$log .= (strlen($title) > 0 ? $title : 'DEBUG')."\n";
	$log .= print_r($data, 1);
	$log .= "\n------------------------\n";
	
	if (!file_exists("./logs/".date('Y-m-d'))) {
        mkdir("./logs/".date('Y-m-d'), 0777, true);
    }

	file_put_contents("./logs/".date('Y-m-d')."/".basename($_SERVER['PHP_SELF'], '.php').".log", $log, FILE_APPEND);
	
	return true;
}

function pauseUs($time = 2000000){
    usleep($time);
}


/**
 * Funo para enviar requisies ao Bitrix utilizando a utenticacaoo passada na request
 * */
function restCommandInstall($method, array $params = Array(), array $auth = Array(), $authRefresh = true)
{
	$queryUrl = $auth["client_endpoint"].$method;
	$queryData = http_build_query(array_merge($params, array("auth" => $auth["access_token"])));

	$curl = curl_init();

	curl_setopt_array($curl, array(
		CURLOPT_POST => 1,
		CURLOPT_HEADER => 0,
		CURLOPT_RETURNTRANSFER => 1,
		CURLOPT_SSL_VERIFYPEER => 1,
		CURLOPT_URL => $queryUrl,
		CURLOPT_POSTFIELDS => $queryData,
	));

	$result = curl_exec($curl);
	curl_close($curl);

	$result = json_decode($result, 1);

	return $result;
}

/**
 * Busca autorizations no BD 
 **/
function getAuthInstall($db){
    
    createDb($db);
    
    $sql = $db->prepare("SELECT * FROM AUTH where CLIENT_ENDPOINT = :CLIENT_ENDPOINT and APP_TOKEN = :APP_TOKEN");
    $sql->bindParam(':CLIENT_ENDPOINT', $_REQUEST["auth"]['client_endpoint']);
    $sql->bindParam(':APP_TOKEN', $_REQUEST["auth"]['application_token']);
    $sql->execute();
    
    if($sql->rowCount() > 0){
        
        if(isset($_REQUEST["auth"]['application_token']) && $_REQUEST["auth"]['application_token'] != ''){
            
            $sql = $db->prepare("UPDATE AUTH set AUTH = :AUTH
                                    WHERE CLIENT_ENDPOINT like :CLIENT_ENDPOINT and APP_TOKEN like :APP_TOKEN");
            $sql->bindParam(':CLIENT_ENDPOINT', $_REQUEST["auth"]['client_endpoint']);
            $sql->bindParam(':APP_TOKEN', $_REQUEST["auth"]['application_token']);
            $sql->bindParam(':AUTH', serialize(array($_REQUEST["auth"]['application_token'] => $_REQUEST["auth"])));
            $sql->execute();
            
            return $_REQUEST["auth"];
            
        }

    }else{

        if(isset($_REQUEST["auth"]['application_token']) && $_REQUEST["auth"]['application_token'] != ''){
        
            $sql = $db->prepare("INSERT into AUTH (CLIENT_ENDPOINT, APP_TOKEN, AUTH)
                                    VALUES (:CLIENT_ENDPOINT, :APP_TOKEN, :AUTH)");
            $sql->bindParam(':CLIENT_ENDPOINT', $_REQUEST["auth"]['client_endpoint']);
            $sql->bindParam(':APP_TOKEN', $_REQUEST["auth"]['application_token']);
            $sql->bindParam(':AUTH', serialize(array($_REQUEST["auth"]['application_token'] => $_REQUEST["auth"])));
            $sql->execute();
            
            return $_REQUEST["auth"];
            
        }
        
    }
    
}

function createDb($db){
    
    require('./src/sqlInstall.php');    
    $sql = $db->prepare($sqlInstall); 
    $sql->execute();
    
}


/**
 * Get new authorize data if you authorize is expire.
 *
 * @param array $auth - Authorize data, received from event
 * @return bool|mixed
 */
function restAuth($auth, $db)
{
	if (!CLIENT_ID || !CLIENT_SECRET)
		return false;

	if(!isset($auth['refresh_token']))
		return false;

	$queryUrl = 'https://oauth.bitrix.info/oauth/token/';
	$queryData = http_build_query($queryParams = array(
		'grant_type' => 'refresh_token',
		'client_id' => CLIENT_ID,
		'client_secret' => CLIENT_SECRET,
		'refresh_token' => $auth['refresh_token'],
	));

	$curl = curl_init();

	curl_setopt_array($curl, array(
		CURLOPT_HEADER => 0,
		CURLOPT_RETURNTRANSFER => 1,
		CURLOPT_URL => $queryUrl.'?'.$queryData,
	));

	$result = curl_exec($curl);
	curl_close($curl);
	
	$result = json_decode($result, 1);
	
	printArray($auth);
	printArray($result);
	
	if(!isset($result['error'])){
	    
	    $result['application_token'] = $auth['application_token'];
	    if(isset($auth['application_token']) && $auth['application_token'] != ''){
	        
            $sql = $db->prepare("UPDATE AUTH set AUTH = :AUTH
                                    WHERE CLIENT_ENDPOINT like :CLIENT_ENDPOINT and APP_TOKEN like :APP_TOKEN");
            $sql->bindParam(':CLIENT_ENDPOINT', $auth['client_endpoint']);
            $sql->bindParam(':APP_TOKEN', $auth['application_token']);
            $sql->bindParam(':AUTH', serialize(array($auth['application_token'] => $result)));
            $sql->execute();

        }
	    
		
	}else{
		$result = FALSE;
	}

	return $result;
}

function restCommand($db, $method, array $params = array(), array $auth = array(), $print = 0 , $authRefresh = true)
{
    
    global $idProcessando;
    global $obsProcessando;

	$curl = curl_init();

	curl_setopt_array($curl, array(
		CURLOPT_POST => 1,
		CURLOPT_HEADER => 0,
		CURLOPT_RETURNTRANSFER => 1,
		CURLOPT_SSL_VERIFYPEER => 1,
		CURLOPT_URL => $auth["client_endpoint"].$method,
		CURLOPT_POSTFIELDS => http_build_query(array_merge($params, array("auth" => $auth["access_token"]))),
	));

	$result = curl_exec($curl);
	curl_close($curl);

	$result = json_decode($result, 1);
	
	if($print == 1){
	    printArray($method);
	    printArray($params);
	    printArray($result);

	}

    // Caso haja erro de token invlido ou expirado, atualiza via restAuth e faz novamente a requisio
	if($authRefresh && isset($result['error']) && in_array($result['error'], array('expired_token', 'invalid_token'))){
		
		$auth = restAuth($auth, $db);
		
		if($auth){
			$result = restCommand($db, $method, $params, $auth, $print, false);
		}
	}elseif(isset($result['error'])){
	    
	    printArray("Error: ".$result['error_description']."<br/>");
        
        writeToLog($result, "Error: ");
        
        updateRequestQueue($idProcessando, 3, $db, 0);
        
        exit();
	    
	}
	
	pauseUs();

	return $result;
}

/**
 * Busca autorizations no BD 
 **/
function getAuth($db){
    $appTkn = APP_TOKEN;
    $sql = $db->prepare("SELECT * FROM AUTH where APP_TOKEN = :APP_TOKEN");
    $sql->bindParam(':APP_TOKEN', $appTkn);
    $sql->execute();
    
    if($sql->rowCount() > 0){
        
        return unserialize($sql->fetch()['AUTH']);

    }else{

        return FALSE;
        
    }
    
}


function getUser($user, $auth, $print = 0){
    
    global $db;
    
    if(isset($user) && $user != ''){
    
        $method = "user.search";
    
        $params = array("filter" => array(
                                        "=email" => trim($user)));  
    
        $result = restCommand($db, $method, $params, $auth[APP_TOKEN], $print, 1);
    
    	return $result;
    }else{
        return false;
    }
}

function addFeedMessage($entity, $id, $message, $auth, $print = 0){
    
    global $db;
    
    $method = 'crm.timeline.comment.add';
    
    $params = array(
            "fields" => array(
                    "COMMENT" => $message, 
                    "ENTITY_TYPE" => $entity,
                    "ENTITY_ID"=> $id,
                ),
        );
    
    $result = restCommand($db, $method, $params, $auth[APP_TOKEN], $print, 1);

}

function getRequest($request){
    
    require("cnx.php");
    
    $sql = $db->prepare("SELECT * FROM new_ecommerce where data like :data");
    $sql->bindParam(":data", $request);
    $sql->execute();
    
    if($sql->rowCount() > 0){
        
        $db = null;
        return false;

    }else{
        $db = null;
        return true;
        
    }
    
}

function getNames($string, $parte) {
    // Divide a string em palavras
    $palavras = explode(' ', $string);

    // Verifica a parte desejada
    switch ($parte) {
        case 0:
            // Retorna a primeira palavra
            return $palavras[0];
        case 1:
            // Retorna a segunda palavra
            if (isset($palavras[1])) {
                return $palavras[1];
            } else {
                return ''; // Se não houver segunda palavra, retorna uma string vazia
            }
        case 2:
            // Retorna a string sem a primeira palavra
            array_shift($palavras);
            return implode(' ', $palavras);
        default:
            return ''; // Retorna uma string vazia para partes não reconhecidas
    }
}

function insertRequest($data){

    require("cnx.php");
    
    $sql = $db->prepare("INSERT INTO new_ecommerce (data, trg, obs, entity) VALUES (:data, :trg, :obs, :entity)");
        $serialized = json_encode($data);                                        
        $sql->bindParam(":data", $serialized);
        $sql->bindParam(":obs", serialize(array("Start")));
        $sql->bindParam(":trg", $_REQUEST['trg']);
        $sql->bindParam(":entity", explode("_", $_REQUEST['trg'])[1]);

        $sql->execute();
        
    $return = array('result' => 'success',
                    'msg' => 'DATA RECEIVED');
    
    $json = json_encode($return, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE );

    echo $json;
    $db = null;    
    
}

function updateRequestQueue($value, $status, $db, $ticketId = null){
    
    if(isset($ticketId) && $ticketId > 0){
        $sql = $db->prepare("UPDATE new_ecommerce set status = :status, bitrix_id = {$ticketId} where id =:id");
    }else{
        $sql = $db->prepare("UPDATE new_ecommerce set status = :status where id =:id");
    }

    $sql->bindParam(':id', $value);
    $sql->bindParam(':status', $status);
    $sql->execute();
    
}

function getFileData($data, $title) {
    // Decodifica o JSON para um array
    // $data = json_decode($json, true);

    // Itera pelos arquivos enviados e encontra o título correspondente
    foreach ($data as $file) {
        if ($file['title'] == $title) {
            // Extrai a extensão do arquivo
            $filePath = $file['absolute_path'][0];
            $extSelf = pathinfo($filePath, PATHINFO_EXTENSION);

            // Obtém a URL pública do arquivo
            $selfie = file_get_contents($file['url'][0]);

            // Cria o array desejado
            $fileData = array("fileData" => array("teseprime".time().".".$extSelf, base64_encode($selfie)));

            return $fileData;
        }
    }

    return null;
}


function getRequestData($db, $entity){

    
    $dados = array();
    
    printArray('SELECT *, (SELECT count(*) FROM new_ecommerce where status = 1 and entity = "'.$entity.'") as iniciados FROM new_ecommerce where status = 0 and entity = "'.$entity.'" order by id limit 5');

    $sql = $db->prepare('SELECT *, (SELECT count(*) FROM new_ecommerce where status = 1 and entity = "'.$entity.'") as iniciados FROM new_ecommerce where status = 0 and entity = "'.$entity.'" order by id limit 5');
    $sql->execute();
    
    if($sql->rowCount() > 0){
        
        $dados = $sql->fetchAll();
        
        return $dados;
        
    }else{
        
        return false;
        
    }
    
}

function getContactId($id, $auth, $print = 0){
    
    global $db;

    $method = "crm.contact.list";

    $params = array("order" => array("ID" => "DESC"),
                    "select" => ["*", "UF_*", "EMAIL", "PHONE"],
                    "filter" => array(
                                    "=UF_CRM_1672173391546" => trim($id),
                                    )
                );  

    $result = restCommand($db, $method, $params, $auth[APP_TOKEN], $print, 1);
    
    if($result['total'] > 0){
        
        return $result['result'][0];
    
    }else{
        
        return false;
        
    }

}

function getContact($cpf, $auth, $print = 0){
    
    global $db;

    $method = "crm.contact.list";

    $params = array("order" => array("ID" => "DESC"),
                    "select" => ["*", "UF_*", "EMAIL", "PHONE"],
                    "filter" => array(
                                    "=UF_CRM_1589202720035" => trim($cpf),
                                    )
                );  

    $result = restCommand($db, $method, $params, $auth[APP_TOKEN], $print, 1);
    
    if($result['total'] > 0){
        
        return $result['result'][0];
    
    }else{
        
        return false;
        
    }

}

function getStatus($forma){
    
    if($forma == "pending"){
        return "Pendente";
    }elseif($forma == "completed"){
        return "Concluído";
    }elseif($forma == "processing"){
        return "Processando";
    }elseif($forma == "cancelled"){
        return "Cancelado";
    }elseif($forma == "failed"){
        return "ERRO";
    }else{
        return $forma;
    }
}

function getDeal($id, $auth, $print = 0){
    
    global $db;

    $method = "crm.deal.list";

    $params = array("order" => array("ID" => "DESC"),
                    "select" => ["*", "UF_*"],
                    "filter" => array(
                                    "=UF_CRM_1708598380" => trim($id),
                                    "CATEGORY_ID" => 0
                                    )
                );  

    $result = restCommand($db, $method, $params, $auth[APP_TOKEN], $print, 1);
    
    if($result['total'] > 0){
        
        return $result['result'][0];
    
    }else{
        
        return false;
        
    }

}

function updateDeal($fields, $id, $auth, $print = 0){
    
    global $db;

    $method = "crm.deal.update";
    
    if(isset($id) && $id >0 ){

        $params = array("fields" => $fields, "ID" => $id);  
    
        $result = restCommand($db, $method, $params, $auth[APP_TOKEN], $print, 1);
        
        return $result;
        
    }else{
            
        return false;
        
    }

}


function updateContact($fields, $id, $auth, $print = 0){
    
    global $db;

    $method = "crm.contact.update";
    
    if(isset($id) && $id >0 ){

        $params = array("fields" => $fields, "ID" => $id);  
    
        $result = restCommand($db, $method, $params, $auth[APP_TOKEN], $print, 1);
        
        return $result;
        
    }else{
            
        return false;
        
    }

}

function addContact($fields, $auth, $print = 0){
    
    global $db;

    $method = "crm.contact.add";

    $params = array("fields" => $fields);  

    $result = restCommand($db, $method, $params, $auth[APP_TOKEN], $print, 1);
    
    return $result;
    

}

function addOrderBitrix($fields, $auth, $print = 0){
    
    global $db;

    $method = "crm.deal.add";

    $params = array("fields" => $fields);  

    $result = restCommand($db, $method, $params, $auth[APP_TOKEN], $print, 1);
    
    return $result;
    

}

function updateOrderBitrix($fields, $id, $auth, $print = 0){
    
    global $db;

    $method = "crm.deal.update";

    $params = array("ID" => $id, "fields" => $fields);  

    $result = restCommand($db, $method, $params, $auth[APP_TOKEN], $print, 1);
    
    return $result;
    

}



// "UF_CRM_61EEF0C9C15A3"  => getField('UF_CRM_61EEF0C9C15A3', $dados['estado'], 'contact', 0, MB_CASE_UPPER), o MB_CASE_UPPER (TUDO MAIÚSCULO) - MB_CASE_TITLE (Primeia Letra Maiúscula)
function getField($field, $item, $entity, $print = 0){
    
    global $db;
    global $auth;
    
    printArray($field);
    printArray($item);
    
    if(trim($item) != ''){
    
        $queryUrl = "crm.{$entity}.userfield.list";
    
        $array = array(
                "filter" => array(
                        "FIELD_NAME" => $field
                    )
            );
    
        $result = restCommand($db, $queryUrl, $array, $auth[APP_TOKEN], $print, 1);
        
        if(isset($result['result'][0]['LIST'][0]['ID'])){
            foreach($result['result'][0]['LIST'] as $key => $value){
                
                $itens[$key] = trim($value['VALUE']);
                $itensId[$key] = $value['ID'];
                        
            }
            $found = array_search(trim($item), $itens);
        }else{
            $found = false;
        }
        
        if($found === false){
    
            updateField($result['result'][0]['ID'], trim($item), $entity, $print);
            $newItem = getField($field,  trim($item), $entity, $print);
            
            return $newItem;
    
        }else{
    
            return $itensId[$found];
        }
    }else{
        // getField($field, 'Não Informado', $entity, $print);
        return "";
    }

}

function updateField($fieldId, $item, $entity, $print =0){
    
    global $db;
    global $auth;
    
    $queryUrl = "crm.{$entity}.userfield.update";
    
    $array = array(
            "ID" => $fieldId,
            "fields" => array(
                        "LIST" => array(array(
                                     "VALUE" => $item
                                    ))
                        ),
        );

    $result = $result = restCommand($db, $queryUrl, $array, $auth[APP_TOKEN], $print, 1);
    sleep(2);
    
}

function adjustPhone($phone){
    
    printArray($phone);
    
    if(!isset($phone) || $phone == ''){
        
        return '';
        
    }
    
    $phone = trim(preg_replace('/[^0-9]/', '', $phone));
    
    if(substr($phone, 0,1) != '+'){
       $phone = '+'.$phone;
    }
    
    printArray($phone);
    
    $phone = str_replace("+55", "", $phone);
    $phone = str_replace("+", "", $phone);
    
    printArray($phone);
    
    if(substr($phone, 0,1) != '0'){
        $phone = '+55'.$phone;
    }else{
        $phone = '+55'.substr($phone, 1);
    }
    
    printArray($phone);
    

    if(strlen($phone) == 14 ){
        return $phone;
        
    }elseif(strlen($phone) == 13 ){
        
        if(substr($phone, 0,1) == '+'){
            
            if(substr($phone, 5,1) == '7' || substr($phone, 5,1) == '8' || substr($phone, 5,1) == '9'){
                
                return substr($phone, 0,5).'9'.substr($phone, 5);
                
            }else{
            
                return $phone;
                
            }
            
        }else{
            return '+'.$phone;
        }
        
    }elseif(strlen($phone) < 13 ){
        
        if(substr($phone, 0,3) != '+55'){
            
            $phone = str_replace("+", "", $phone);
            
            if(strlen($phone) == 11){
                
                return '+55'.$phone;
                
            }elseif(strlen($phone) < 11){
                if(substr($phone, 2,1) == '7' || substr($phone, 2,1) == '8' || substr($phone, 2,1) == '9'){
                    return '+55'.substr($phone, 0,2).'9'.substr($phone, 2);
                }else{
                    return '+55'.$phone;
                }
            }else{
            
                return $phone;
            
            }
        }else{
            
            return $phone;
            
        }
        
        
        
    }else{    
        return $phone;
    }
    
}

function validateEmail($email) {
    
    $email = explode(";", $email)[0];
    
    if(!filter_var($email, FILTER_VALIDATE_EMAIL) && trim($email) != '') {

        echo "E-mail ".$email." inválido";
        writeToLog("E-mail ".$email." inválido", "ERRO ".date('Y-m-d H:i:s'));

        return "";
    }else{
        return $email;
    }
}

function phonesUpdate($oldPhones, $newPhone, $type = 'WORK', $phonEmail = 'PHONE'){
    
    printArray($oldPhones);
    
    if(isset($newPhone) && $newPhone != ''){
        
        if($phonEmail == 'PHONE'){
    
            $found = false;
            foreach($oldPhones as $key => $value){
                if(adjustPhone($value['VALUE']) == adjustPhone($newPhone)){
                    $found = true; 
                    
                    $oldPhones[$key]['VALUE'] = adjustPhone($value['VALUE']);
                }else{
                    
                    $oldPhones[$key]['VALUE'] = adjustPhone($value['VALUE']);
                }
            }
            
            if($found == false){
                
                $oldPhones[] = array(
                                'VALUE_TYPE' => $type,
                                'VALUE'      => adjustPhone($newPhone)
                    );
                
            }
            
            return $oldPhones;
            
        }elseif($phonEmail == 'EMAIL'){
            
            $found = false;
            foreach($oldPhones as $key => $value){
                if($value['VALUE'] == validateEmail($newPhone)){
                    $found = true;        
                }
            }
            
            if($found == false){
                
                $oldPhones[] = array(
                                'VALUE_TYPE' => $type,
                                'VALUE'      => validateEmail($newPhone)
                    );
                
            }
            
            return $oldPhones;
            
        }else{
            
            $found = false;
            foreach($oldPhones as $key => $value){
                if($value['VALUE'] == $newPhone){
                    $found = true;        
                }
            }
            
            if($found == false){
                
                $oldPhones[] = array(
                                'VALUE_TYPE' => $type,
                                'VALUE'      => $newPhone
                    );
                
            }
            
            return $oldPhones;
            
        }
        
    }else{
        return $oldPhones;
    }
    
}


/*******************************************************************************/



